jQuery.sap.require(
    "sap.m.MessageToast",
    "sas.modelfactory.proxies.TemplateProxy",
    "sas.modelfactory.util.PreferencesUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.controller("sas.modelfactory.project.settings.ProjectModel", {

    projectUri: null,
    project: null,
    blankModel: null,
    isNewModel: false,
    newModelDialog: null,
    dataLoaded: false,
    templateModel: null,
    preferenceId: null,
    profileBuilt: false,
    errorMsg: null,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.sas.modelfactory.project.settings.ProjectModel
     */
    onInit: function() {
        sas.eventBus.subscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.subscribe("ProjectEvent", "profileState", this.updateBuildProfileState, this);

        var controller = this;

        this.getView().addEventDelegate({
            onBeforeFirstShow: function() {
                logUtil.log("ProjectModel onBeforeFirstShow hit");
                controller.getView().setHeight("100%");
            },

            onBeforeShow: function() {
                logUtil.log("ProjectModel onBeforeShow hit");
                controller.initializeView();
            }

        });

        this.getView().table.attachAfterColumnsDialogClose(function(oEvent) {
            // If OK was pressed, persist the column manager changes
            if (oEvent.getParameter("acceptedChange"))
                controller.onColumnManagerClose();
        });
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.project.settings.ProjectModel
     */
    onBeforeRendering: function() {

    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.project.settings.ProjectModel
     */
    onAfterRendering: function() {

        //jQuery(".modelFactoryMain-buttonHBox").parent().css("float", "right");
    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.project.settings.ProjectModel
     */
    onExit: function() {
        logUtil.log("sas.modelfactory.project.settings.ProjectModel: onExit called");
        sas.eventBus.unsubscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.unsubscribe("ProjectEvent", "profileState", this.updateBuildProfileState, this);
    },

    updateBuildProfileState: function(sChannelId, sEventId, oData) {
        var controller = this;
        if (oData){
            controller.profileBuilt = oData.profileBuilt;
        }
    },

    handleProjectLoadedEvent: function(channelId, eventId, data) {
        var controller = this;
        if (data){
            controller.profileBuilt = data.profileBuilt;

            controller.initializeView();
        }
    },

    initializeView : function() {
        var controller = this;

        // Get the current open project
        var model = sap.ui.getCore().getModel("openProject");
        var project = null;
        if (model){
            project = model.getProperty("/project");
            controller.profileBuilt = model.getProperty("/profileBuilt");
        }

        if (project) {
            controller.project = project;

            // Get the project level templates
            if (controller.project){
                controller.getTemplateList(controller.project);
            }

            this.updateMessages();
        }
    },

    getTemplateList: function(project) {
        var model = new sap.ui.model.json.JSONModel();
        var view = this.oView;
        var controller = this;

        var onSuccess = function(oJson){
            // This code removes hidden templates such as the blank template
            var i = 0;
            while ( i<oJson.items.length) {
                if (oJson.items[i].hidden) {
                    oJson.items.splice(i,1);
                } else {
                    i++;
                }
            }

            if (!controller.dataLoaded){
                controller.dataLoaded = true;
                model.setData({templateData: oJson.items});
                view.setAndBindModel(model, "/templateData");
            } else {
                // Update the data
                view.table.getModel().setProperty("/templateData", oJson.items);
            }

            var callback = function(preferenceId) {
                controller.preferenceId = preferenceId;
            }

            // Update table with column preferences
            sas.modelfactory.util.PreferencesUtil.getColumnPref(controller.project.id, "FactoryMiner.ModelTempColumnPreferences", view.table, callback);
        };

        var onComplete = function(oJson){
            view.setBusy(false);
        };

        templateProxy.getProjectTemplates(project.id, onSuccess, null, onComplete);
        view.setBusy(true);
    },

    onLinkPress: function(link) {
        var controller = this;
        var id = null;
        if (link.data()){
            id = link.data().id;
        }
        this.isNewModel = false;
        if (id) {
            controller.getProjectTemplate(controller.project.id, id);
        }
    },

    onViewPropertiesPress: function(evt) {
        var controller = this;
        var index = this.oView.table.getSelectedIndex();
        var context = this.oView.table.getContextByIndex(index);
        this.isNewModel = false;
        if (context) {
            var selectedData = context.getObject();
            logUtil.log(selectedData);

            controller.getProjectTemplate(controller.project.id, selectedData.id);
        }
    },

    removeTemplates: function(items) {
        var controller = this;
        var view = this.oView;
        var model = sap.ui.getCore().getModel("openProject");
        var project = null;
        if (model){
            project = model.getProperty("/project");
        }

        var removeTemplate = function(items){
            if (items && items.length > 0){
                var selectedData = items.pop();

                var onSuccess = function(oJson){

                };

                var onError = function(xhr, err, msg){
                    logUtil.log("Error deleting project model template");
                    var json = xhr.responseJSON;
                    var errMsg = sas.modelfactory.RB.getText("projectModelView.error.deleteProjectModels.txt") +
                        "\n\n" + sas.modelfactory.RB.getText("projectModelView.error.status.txt") + xhr.status +
                        "\n\n " + json.message;
                    sas.hc.m.MessageBox.error(errMsg);

                    if (items.length > 0){
                        removeTemplate(items);
                    }
                };

                var onComplete = function(oJson){
                    if (items.length > 0){
                        removeTemplate(items);
                    } else if (items.length === 0){
                        controller.getTemplateList(project);
                        busySpinner.show(false);
                    }
                };

                if (project){
                    templateProxy.removeProjectModelTemplate(project.id, selectedData.id, onSuccess, null, onComplete);
                }
            }
        };

        removeTemplate(items);

        // Create a busy spinner
        busySpinner.show(true,
            sas.modelfactory.RB.getText("projectModelView.busyMsg.deleteTitle.txt"),
            sas.modelfactory.RB.getText("projectModelView.busyMsg.deleteMsg.txt"));
    },

    onRemoveModelTemplatePress: function(evt) {
        var controller = this;
        var view = this.oView;
        var indices = view.table.getSelectedIndices();
        var items = [];
        for (var i=0; i<indices.length; i++){
            var context = view.table.getContextByIndex(indices[i]);
            if (context) {
                var item = context.getObject();
                if (!item.modelsHaveBeenRunFlag && item.miningAlgorithm.internalName !== "Baseline"){
                    items.push(item);
                }
            }
        }

        var confirmDialog =  sap.ui.jsfragment("sas.modelfactory.dialog.ConfirmDialog", this);
        confirmDialog.initDialog(sas.modelfactory.RB.getText("projectSettingsModel.confirmDeleteProject.title.txt"),
            sas.modelfactory.RB.getText("projectSettingsModel.confirmDeleteProject.txt"),
            controller.removeTemplates,
            function(){
                // Nothing to do here
            },
            items);
        confirmDialog.setInitialFocus(confirmDialog.getEndButton());
        confirmDialog.open();

    },

    getProjectTemplate: function (projId, templateId)
    {
        var controller = this;
        var view = this.oView;

        var onSuccess = function (oJson)
        {
            logUtil.log("Project Template properties: ", oJson);

            var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", controller);
            var title = sas.modelfactory.RB.getText(oJson.displayName);
            dialog.setTitle(title);
            if (oJson.miningAlgorithm.internalName !== "Baseline"){
                if (controller.project.buildProfileAvailable) {
                    dialog.addSaveButton();
                }
                dialog.addSaveAsButton();
            }
            dialog.open();
            // publish to event bus
            sas.eventBus.publish("ModelTemplateEvent", "editTemplate", {
                template: oJson});

        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.modelTemplate.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        // Get the model template data
        if (projId && templateId) {
            templateProxy.getProjectTemplate(projId, templateId, onSuccess, onError);
        }
    },

    onNewTemplateFromAlgorithmPress: function(evt) {
        var controller = this;
        controller.isNewModel = true;

        // Get the list of algorithms for display
        var algorithms = sap.ui.getCore().getModel("algorithms").getData().algorithmsForDisplay;

        var onSuccess = function (oJson) {
            controller.newModelDialog = sap.ui.jsfragment("sas.modelfactory.dialog.NewModelTemplateDialog", controller);

            // fetch the blank template- The algorithms are the in the properties where groupName = "Model"
            // compile a unique list
            var templates = oJson.items;

            var onBlankTemplateSuccess = function (oJson) {
                controller.blankModel = oJson;

                var model = new sap.ui.model.json.JSONModel();
                model.setData({algorithms: algorithms});
                controller.newModelDialog.setModel(model);
                controller.newModelDialog.open();
                sas.eventBus.publish("ModelTemplateEvent", "newModelTemplate", {template: oJson, saveEnabled: true});
            };

            var onBlankTemplateError = function(xhr, err, msg){
                var errMsg = sas.modelfactory.RB.getText("Error loading blank model template");
                var title = sas.modelfactory.RB.getText("Loading template");
                sas.hc.m.MessageBox.error(errMsg, title);
            };

            for (var i=0; i<templates.length; i++) {
                logUtil.log("name: = " + templates[i].name);
                if (templates[i].name === "Blank Template") {
                    var links = templates[i].links;
                    messageProxy.request(links, "self", "GET", "json", null, onBlankTemplateSuccess, onBlankTemplateError);
                    break;
                }
            }
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.loading.desc.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.loading.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        }

        var onComplete = function () {
            sas.eventBus.unsubscribe("ModelTemplateEvent", "dataLoaded", this.modelDataLoadedHandler, this);
        }


        templateProxy.getTemplates(onSuccess, onError, onComplete);
    },

    onAddTemplateFromGlobalTemplatePress: function(evt) {

        var controller = this;
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.AddModelTemplateDialog", controller);
        var model = new sap.ui.model.json.JSONModel();

        dialog.open();
        // load the available templates
        var onSuccess = function (oJson) {

            // This code removes hidden templates such as the blank template
            var i = 0;
            while ( i<oJson.items.length) {
                if (oJson.items[i].hidden) {
                    oJson.items.splice(i,1);
                } else {
                    i++;
                }
            }
            templateModel = new sap.ui.model.json.JSONModel();
            templateModel.setData({templateData: oJson.items});

            sas.eventBus.publish("newModel", "addModelToProject", {templateData:  oJson.items});
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.modelTemplate.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        }

        var onComplete = function () {

        }


        var rootModel = sap.ui.getCore().getModel("root");
        if (rootModel === undefined) {
            logUtil.error("Root Link Error: Root model undefined.");
        } else {
            var links = rootModel.getProperty("/root");
            // Get the global template list
            messageProxy.request(links, "templates", "GET", "json", null, onSuccess);
        }
    },

    onAddModelTemplateCreate: function(selectedTemplates) {
        logUtil.log("AddModelTemplatesCreate: ", selectedTemplates);
        var controller = this;
        var view = this.getView();

        // Find the run button object
        var runStopButton = null;
        var mainView = null;
        var objectContentPage = sap.ui.getCore().byId(controller.getView().data("objectContentPageId"));
        if (objectContentPage) {
            mainView = objectContentPage.getParent();
            if (mainView)
                runStopButton = mainView.runStopButton;
        };

        var createTemplate = function(items) {
            if (items && items.length > 0){
                var selectedTemplate = items.pop();

                var onGetTemplateSuccess = function(oJson) {
                    var template = oJson;

                    var createRenamedTemplate = function(renamedTemplate){
                        //var diagModel = dialog.getModel();
                        //var renamedTemplate = diagModel.oData.template;

                        var onCreateSuccess = function(oJson) {
                            logUtil.log("Success");

                            if (items.length > 0){
                                createTemplate(items);
                            }
                        };

                        var onCreateError = function(xhr, err, msg){
                            var errResponse = jQuery.parseJSON(xhr.responseText);
                            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.addModel.title.txt");
                            sas.hc.m.MessageBox.error(errResponse.message, title);
                        };

                        var onCreateComplete = function(oJson) {
                            logUtil.log("Complete");
                            if (items.length === 0){
                                controller.oView.setBusy(false);
                                controller.getTemplateList(controller.project);

                                // Enable the Run button when done creating the model
                                if (runStopButton)
                                    runStopButton.setEnabled(true);
                            }
                        };

                        templateProxy.addModelTemplateToProject(controller.project.id, renamedTemplate, onCreateSuccess, onCreateError, onCreateComplete);
                    }

                    // Create a unique name
                    var displayName = template.displayName;
                    var model = view.table.getModel();
                    var maxNum = 0;
                    var found = false;
                    if (model){
                        var list = model.getProperty("/templateData");
                        for(var j=0; j<list.length;j++){
                            var str = list[j].displayName;
                            if (str.search(template.displayName) !== -1){
                                found = true;
                                var parts = str.split(" ");
                                if (parts.length > 0) {
                                    var thisNum = parseInt(parts[parts.length-1]);
                                    if (!isNaN(thisNum) && thisNum > maxNum)
                                        maxNum = thisNum;
                                }
                            }
                        }
                    }
                    if (found){
                        maxNum++;
                        displayName = template.displayName + " " + maxNum;
                    }

                    template.displayName = displayName;
                    createRenamedTemplate(template);

                    /*var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.RenameDialog", this);
                    dialog.renameHandler = createRenamedTemplate;

                    var dialogModel = new sap.ui.model.json.JSONModel();
                    dialogModel.setData({template: template,
                        templateData: controller.oView.table.getModel().oData.templateData});
                    dialog.bindElement("/template");
                    dialog.uniqueNamePath = "/templateData";
                    dialog.setNameProperty("displayName");
                    dialog.setModel(dialogModel);
                    dialog.setTitle(sas.modelfactory.RB.getText("renameDialog.name.title.txt"));
                    dialog.setName(displayName);
                    dialog.cancelCallback = function(){
                        if (items.length === 0){
                            controller.oView.setBusy(false);
                            controller.getTemplateList(controller.project);

                            // Enable the Run button when done creating the model
                            if (runStopButton)
                                runStopButton.setEnabled(true);
                        }
                    };

                    dialog.open();*/
                };

                var onGetTemplateError = function(xhr, err, msg){

                };

                var onGetTemplateComplete = function(oJson) {

                };

                if (selectedTemplate){
                    logUtil.log("Add template:", selectedTemplate.name);
                    templateProxy.getTemplate(selectedTemplate.id, onGetTemplateSuccess, onGetTemplateError, onGetTemplateComplete);
                }
            }
        };

        if (selectedTemplates.length > 0){

            // Disable the Run button while creating the model
            if (runStopButton)
                runStopButton.setEnabled(false);

            createTemplate(selectedTemplates);
            controller.oView.setBusy(true);
        }
    },

    onNewModelTemplateCreate: function(name, dialog) {
        var selectedItem = dialog.modelAlgorithm.getSelectedItem();
        logUtil.log("onNewModelTemplateCreate: ", this);
        logUtil.log("onNewModelTemplateCreate: ", name.getValue());
        logUtil.log("onNewModelTemplateCreate: ", selectedItem.getText());
        logUtil.log("onNewModelTemplateCreate: ", selectedItem.getKey());

        var controller = this;

        // Find the selected algorithm index.
        var algorithms = selectedItem.getModel().getData().algorithms;
        var algorithmIndex = -1;
        for (var i=0; i< algorithms.length; i++){
            if (algorithms[i].id === selectedItem.getKey()){
                algorithmIndex = i;
                break;
            }
        }

        // initialize the blankModel and "use" the respective model properties component
        var newModel = controller.blankModel;
        newModel.name = name.getValue();
        newModel.displayName = name.getValue();
        newModel.createdBy = null;
        newModel.modifiedBy = null;
        newModel.createdTimestamp = null;
        newModel.modifiedTimestamp = null;
        newModel.links = null;
        newModel.description = "User created model";
//        newModel.algorithmName = algorithms[algorithmIndex].displayName;
        newModel.miningAlgorithm = {id: algorithms[algorithmIndex].id, internalName: algorithms[algorithmIndex].internalName};
        newModel.favorite = dialog.defaultCheckBox.getSelected();
        newModel.makeGlobalTemplate = dialog.availCheckBox.getSelected();
        newModel.hidden = false;
        newModel.sasonly = false;
        for (var i=0; i<newModel.properties.length; i++) {
            if (newModel.properties[i].groupName === 'Model') {
                if (newModel.properties[i].componentName === algorithms[algorithmIndex].internalName) {
                    if (newModel.properties[i].propertyName === '_use') {
                        newModel.properties[i].propertyValue = 'Y';
                        newModel.properties[i].position = 1;
                    }
                }
            }
        }

        // open the model dialog
        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.ModelTemplateDialog", this);
        var title =  name.getValue();
        dialog.setTitle(title);
        dialog.addSaveButton();
        dialog.open();
        sas.eventBus.publish("ModelTemplateEvent", "newModelTemplate", {template: newModel, saveEnabled: true});

    },

    onTemplateSave: function(dialog) {
        var controller = this;
        var view = this.getView();
        var model = dialog.getModel();
        logUtil.log("Save model template:", JSON.stringify(model.oData.template));

        // Find the run button object
        var runStopButton = null;
        var mainView = null;
        var objectContentPage = sap.ui.getCore().byId(controller.getView().data("objectContentPageId"));
        if (objectContentPage) {
            mainView = objectContentPage.getParent();
            if (mainView)
                runStopButton = mainView.runStopButton;
        };

        var onSuccess = function (oJson) {
            // Templated updated, refresh the template list.
            if (controller.project) {
                controller.getTemplateList(controller.project);
                controller.reloadProject();
            }
        };

        var onError = function(xhr, err, msg){
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.updateModelTemplate.txt");
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function(oJson){
            // Enable the Run button when done creating the model
            if (runStopButton)
                runStopButton.setEnabled(true);

            view.setBusy(false);
        };

        controller.oView.setBusy(true);

        // Disable the Run button while creating the model
        if (runStopButton)
            runStopButton.setEnabled(false);

        // Update or create the model template
        if (controller.isNewModel) {
            model.oData.template.name = model.oData.template.displayName;
            var makeGlobalTemplate = model.oData.template.makeGlobalTemplate;
            delete model.oData.template['makeGlobalTemplate'];
            templateProxy.createModelTemplateForProject(controller.project.id, model.oData.template, onSuccess, null, onComplete);

            if (makeGlobalTemplate) {
                var onGlobalSuccess = function (oJson) {
                    logUtil.log("Create global template")
                };
                var onGlobalError = function(xhr, err, msg){
                    logUtil.log("create global template failed");
                    var json = xhr.responseJSON;
                    var errMsg = sas.modelfactory.RB.getText("projectModelView.error.createGlobalTemplate.txt") +
                        "\n\n" + sas.modelfactory.RB.getText("projectModelView.error.status.txt") + xhr.status +
                        "\n\n " + json.message;
                    sas.hc.m.MessageBox.error(errMsg);
                };
                var onGlobalComplete = function () {
                    logUtil.log("Create global template completed");
                };
                templateProxy.createGlobalModelTemplate (model.oData.template, onGlobalSuccess, onGlobalError, onGlobalComplete);
            }
        } else {
            templateProxy.updateModelTemplateForProject(controller.project.id, model.oData.template, onSuccess);
        }
    },

    onTemplateSaveAs: function(dialog) {
        var controller = this;
        var model = dialog.getModel().oData.template;
        logUtil.log("Save model template:", JSON.stringify(model));


        // Update or create the model template

        var dialog = sap.ui.jsfragment("sas.modelfactory.dialog.RenameDialog", this);
        dialog.renameHandler = this.saveAsTemplateRenameHandler;

        var dialogModel = new sap.ui.model.json.JSONModel();
        dialogModel.setData({template: model,
            templateData: controller.oView.table.getModel().oData.templateData});
        dialog.bindElement("/template");
        dialog.uniqueNamePath = "/templateData";
        dialog.setNameProperty("displayName");
        dialog.setModel(dialogModel);

        dialog.open();
    },

    saveAsTemplateRenameHandler: function(dialog) {

        var view = this.getView();
        var controller = this;
        var model = dialog.getModel();
        model.oData.template.name = model.oData.template.displayName;
        controller.oView.setBusy(true);

        var onSuccess = function (oJson) {
            // Templated updated, refresh the template list.
            if (controller.project) {
                controller.getTemplateList(controller.project);
            }
        };
        var onError = function(xhr, err, msg){
            var errResponse = jQuery.parseJSON(xhr.responseText);
            var msg;
            if (errResponse) {
                msg = errResponse.message;
            }
            var errMsg = sas.modelfactory.RB.getText("viewModelTemplate.error.saveAsModelTemplate.txt" + "\n\n" + msg);
            var title = sas.modelfactory.RB.getText("viewModelTemplate.error.title.txt");
            sas.hc.m.MessageBox.error(errMsg, title);
        };

        var onComplete = function(oJson){
            view.setBusy(false);
        };

        model.oData.template.name = model.oData.template.displayName;
        var makeGlobalTemplate = model.oData.template.makeGlobalTemplate;
        delete model.oData.template['makeGlobalTemplate'];
        templateProxy.createModelTemplateForProject(controller.project.id, model.oData.template, onSuccess, onError, onComplete);

    },

    renameHandler: function(dialog) {
        var controller = this;
        var model = dialog.getModel();
        model.oData.template.name = model.oData.template.displayName;

        controller.oView.setBusy(true);

        var onSuccess = function() {
            controller.getTemplateList(controller.project);
        };

        var onComplete = function(oJson){
            controller.view.setBusy(false);
        };

        templateProxy.createModelTemplateForProject(controller.project.id, model.oData.template, onSuccess);
    },

    resetButton: function() {},

    handleMetadataRowSelectionChange: function(oEvent) {
        var view = this.getView();
        var index = view.table.getSelectedIndex();
        logUtil.log(index);
        var oneRowSelected = view.table.getSelectedIndices().length === 1;
        var model = sap.ui.getCore().getModel("openProject");
        var project = null;
        if (model){
            project = model.getProperty("/project");
        }

        if (index !== -1) {
            var selectedTemplate = view.table.getContextByIndex(index).getObject();
            var isBaseline = selectedTemplate.miningAlgorithm.internalName === "Baseline";
            this.oView.viewPropBtn.setEnabled(oneRowSelected && !isBaseline);
        }

        var canDelete = function(indices){
            for (var i=0;i<indices.length;i++){
                var temp = view.table.getContextByIndex(indices[i]).getObject()
                if (temp.miningAlgorithm.internalName !== "Baseline" && !temp.modelsHaveBeenRunFlag){
                    return true;
                }
            }
            return false;
        };
        this.oView.removeModelTemplate.setEnabled(canDelete(this.oView.table.getSelectedIndices()));
    },

    onColumnMove: function(evt) {
        logUtil.log("Results table column moved:", evt.getParameter("column").sId, "moved to", evt.getParameter("newPos"));

        // A column has moved
        var params = evt.getParameters();
        this.columnsMoved(params);
    },

    columnsMoved: function(params) {
        logUtil.log("Columns changed.  Do something with columns.")
        if (params) {
            var controller = this;
            var colMoved = params.column;
            var pos = params.newPos;
            var table = controller.getView().table;
            var cols = null;
            if (table){
                cols = table.getColumns();
            }

            // Update the columms
            if (table && cols){
                for(var i=0;i<cols.length;i++){
                    if (cols[i].getId() === colMoved.getId()){
                        cols.splice(i, 1);
                        break;
                    }
                }
                cols.splice(pos, 0, colMoved);
            }
            table.removeAllColumns();
            for(var j=0;j<cols.length;j++){
                table.addColumn(cols[j]);
            }

            var storeSuccess = function(args) {
                controller.preferenceId = args.id;
            }

            // Store column preferences
            sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, "FactoryMiner.ModelTempColumnPreferences", "modTemp_", table, controller.preferenceId, storeSuccess);
        }
    },

    onColumnManagerClose: function() {
        var controller = this;
        var table = controller.getView().table;

        var storeSuccess = function(args) {
            controller.preferenceId = args.id;
        }

        // Store column preferences
        sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, "FactoryMiner.ModelTempColumnPreferences", "modTemp_", table, controller.preferenceId, storeSuccess);
    },

    updateMessages: function() {
        // update error message
        var controller = this;
        this.errorMsg = [];

        // Check for any errors...
        if (!controller.profileBuilt) {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.noProfile.txt");
            this.errorMsg.push(errorStr);
        }

        if (this.project.profileStatus.key === "MODIFIED") {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.modifiedProfile.txt");
            this.errorMsg.push(errorStr);
        }

        // Update the error display
        if (this.errorMsg.length  > 0) {
            var errorTxt = this.buildErrorMessage();
            var warningList = this.getView().msgList;
            if (warningList){
                warningList.removeAllItems();
                warningList.setVisible(true);
                var list = this.buildErrorList();
                for (var i = 0; i < list.length; i++) {
                    warningList.addItem(list[i]);
                }
            }

            var top = this.getView().pageLayout.getTop();
            var size = 48 + 30 + this.errorMsg.length * 40;
            size += "px";
            top.setSize(size);
        } else {
            this.getView().msgList.setVisible(false);
            var top = this.getView().pageLayout.getTop();
            top.setSize("48px");

            this.getView().msgList.setVisible(false);
        }
    },

    buildErrorMessage: function(){
        var controller = this;
        var errorTxt = sas.modelfactory.RB.getText("projectProfileView.errorMsg.txt") + "\n\n";
        for (var i=0;i<this.errorMsg.length;i++){
            errorTxt += "\u2022 " + this.errorMsg[i] + "\n";
        }
        return errorTxt;
    },

    buildErrorList: function(){
        var controller = this;
        var msgList = [];
        for (var i=0;i<this.errorMsg.length;i++){
            var item = new sas.hc.m.StandardListItem({
                title: this.errorMsg[i],
                icon: sas.icons.HC.ALERT
            }).addStyleClass("ProjectDataView-warning");
            msgList.push(item);
        }
        return msgList;
    },

    reloadProject: function () {
        var links = this.project.links;
        var projectUri = messageProxy.findUri(links, "self", "GET");
        if (projectUri) {
            // publish to event bus
            sas.eventBus.publish("ProjectEvent", "reload", { projectUri: projectUri});
        }
    }
});