jQuery.sap.require(
    "sas.hc.m.MessageBox",
    "sas.modelfactory.proxies.ProjectProxy",
    "sas.modelfactory.proxies.SegmentsProxy",
    "sas.modelfactory.proxies.MessageProxy",
    "sas.modelfactory.util.PreferencesUtil",
    "sas.modelfactory.util.SpinnerUtil",
    "sas.modelfactory.util.LogUtil"
);

sap.ui.controller("sas.modelfactory.project.settings.ProjectProfile", {

    project: null,
    profileBuilt: false,
    loaded: false,
    isPolling: false,
    stopPolling: false,
    CONST_POLL_TIME: 5000, // 5 secs.
    errorMsg: null,
    preferenceId: null,

    /**
     * Called when a controller is instantiated and its View controls (if available) are already created.
     * Can be used to modify the View before it is displayed, to bind event handlers and do other one-time initialization.
     * @memberOf sas.sas.modelfactory.project.settings.ProjectProfile
     */
    onInit: function() {
        sas.eventBus.subscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.subscribe("ProjectEvent", "profileState", this.updateBuildProfileState, this);

        var controller = this;

        this.getView().addEventDelegate({
            onBeforeFirstShow: function() {
                logUtil.log("ProjectProfile onBeforeFirstShow hit");
                controller.getView().setHeight("100%");
            },

            onBeforeShow: function() {
                logUtil.log("ProjectProfile onBeforeShow hit");
                controller.stopPolling = false;
                controller.initializeView();
            },

            onBeforeHide: function() {
                logUtil.log("ProjectProfile onBeforeHide hit");
                controller.stopPolling = true;
            }
        });

        this.getView().table.attachAfterColumnsDialogClose(function(oEvent) {
            // If OK was pressed, persist the column manager changes
            if (oEvent.getParameter("acceptedChange"))
                controller.onColumnManagerClose();
        });
    },

    /**
     * Similar to onAfterRendering, but this hook is invoked before the controller's View is re-rendered
     * (NOT before the first rendering! onInit() is used for that one!).
     * @memberOf sas.modelfactory.project.settings.ProjectProfile
     */
    onBeforeRendering: function() {

    },

    /**
     * Called when the View has been rendered (so its HTML is part of the document). Post-rendering manipulations of the HTML could be done here.
     * This hook is the same one that SAPUI5 controls get after being rendered.
     * @memberOf sas.modelfactory.project.settings.ProjectProfile
     */
    onAfterRendering: function() {
        //jQuery(".actionButton-floatRight").parent().css("float", "right");
    },

    saveModel: function(model) {
        return sas.ajax({
            url: ROOT + MODEL,
            type: "PUT",
            data: model
        });
    },

    /**
     * Called when the Controller is destroyed. Use this one to free resources and finalize activities.
     * @memberOf sas.modelfactory.project.settings.ProjectProfile
     */
    onExit: function() {
        logUtil.log("sas.modelfactory.project.settings.ProjectProfile: onExit called");

        // Stop any polling
        this.stopPolling = true;

        sas.eventBus.unsubscribe("ProjectEvent", "projectLoaded", this.handleProjectLoadedEvent, this);
        sas.eventBus.unsubscribe("ProjectEvent", "profileState", this.updateBuildProfileState, this);
    },

    updateBuildProfileState: function(sChannelId, sEventId, oData) {
        var controller = this;
        logUtil.log(controller.getView().getId());
        if (oData){
            controller.buildProfileAvailable = oData.buildProfileAvailable;
            controller.profileBuilt = oData.profileBuilt;

            // If the profile is not built but are segments are loaded, the profile
            // has been updated. Time to reload.
            if (!controller.profileBuilt && controller.loaded){
                controller.handleProfileBuiltEvent();
            }
        }
    },

    handleProfileBuiltEvent : function(channelId, eventId, data) {
        var view = this.getView();
        var controller = this;

        // Clear any existing columns
        view.table.removeAllColumns();

        // Clear column prefs
        if (controller.project){
            var key = "FactoryMiner.ProfileColumnPreferences.";
            if (controller.getTargetType() === "INTERVAL")
                key = key + "Numeric";
            else
                key = key + "Class";
            sas.modelfactory.util.PreferencesUtil.deleteColumnPrefs(controller.project.id, key);
        }

        // Need to reload the profile table
        this.loaded = false;
        view.table.getModel(null);
        view.table.unbindRows();
    },

    handleProjectLoadedEvent: function(channelId, eventId, data) {
        var controller = this;
        if (data){
            controller.buildProfileAvailable = data.buildProfileAvailable;
            controller.profileBuilt = data.profileBuilt;

            controller.initializeView();
        }
    },

    initializeView : function() {
        var controller = this;

        // Get the current open project
        var model = sap.ui.getCore().getModel("openProject");
        var project = null;
        if (model){
            project = model.getProperty("/project");
            controller.buildProfileAvailable = model.getProperty("/buildProfileAvailable");
            controller.profileBuilt = model.getProperty("/profileBuilt");
        }

        if (project) {
            controller.project = project;

            // Get the segments for the project
            if (controller.project && controller.profileBuilt){
                controller.getSegments(controller.project);
            }

            controller.updateMessages();
        }
    },

    onRowSelectionClick: function (evt)
    {
        this.refreshToolbar();
    },

    refreshToolbar: function () {
        var indices = this.oView.table.getSelectedIndices();
        logUtil.log("rows selected: " + indices);

        // Enable/disable the actions button
        if (indices.length == 0) {
            this.byId("excludeBtn").setEnabled(false);
            this.byId("includeBtn").setEnabled(false);
            //this.byId("exploreBtn").setEnabled(false);
        }
        else if (indices.length == 1) {
            var context = this.oView.table.getContextByIndex(indices[0]);
            if (context) {
                var selectedData = context.getObject();

                // If excluded, enable include and disable exclude
                if (selectedData.excluded) {
                    this.byId("includeBtn").setEnabled(true);
                    this.byId("excludeBtn").setEnabled(false);
                }
                // Otherwise
                else {
                    this.byId("includeBtn").setEnabled(false);
                    this.byId("excludeBtn").setEnabled(true);
                }
            }
        }
        else if (indices.length > 1) {
            var excludeEnabled = false;
            var includeEnabled = false;
            var context;
            var selectedData;

            // Cycle through the list of segments
            for (var i = 0; i < indices.length; i++) {
                context = this.oView.table.getContextByIndex(indices[i]);
                if (context) {
                    selectedData = context.getObject();
                    // If excluded, enable include.  Otherwise, enable exclude
                    if (selectedData.excluded)
                        includeEnabled = true;
                    else
                        excludeEnabled = true;
                }
            }

            this.byId("excludeBtn").setEnabled(excludeEnabled);
            this.byId("includeBtn").setEnabled(includeEnabled);
        }
    },

    getSegments: function(proj) {

        var model = new sap.ui.model.json.JSONModel();
        var view = this.oView;
        var controller = this;

        var handlePreProcessingJson = function(oJson){
            logUtil.log("getSegments response segment count: ", (oJson.items.length));

            if (oJson){
                var data = oJson.items;
                for (var i=0;i<data.length;i++){
                    var value = data[i].excluded?sas.modelfactory.RB.getText("projectProfileView.menuItem.exclude.txt"):sas.modelfactory.RB.getText("projectProfileView.menuItem.include.txt");
                    data[i].excludedText = value;
                }
            }

            if (!controller.loaded){
                controller.loaded = true;
                busySpinner.show(false);
                model.setData({preData: oJson.items});
                view.setAndBindModel(model, "/preData");
            } else {
                // Update the data
                view.table.getModel().setProperty("/preData", oJson.items);
            }

            // Update table with column preferences
            var key = "FactoryMiner.ProfileColumnPreferences.";
            if (controller.getTargetType() === "INTERVAL")
                key = key + "Numeric";
            else
                key = key + "Class";

            var callback = function(preferenceId) {
                controller.preferenceId = preferenceId;
            }

            sas.modelfactory.util.PreferencesUtil.getColumnPref(controller.project.id, key, view.table, callback);

            // If not already started, poll model results
            if (!controller.isPolling) {
                controller.pollRunningSegments(proj);
                controller.pollRunningSegments(proj);
            }
        };

        var handlePreProcessingComplete = function(oJson) {
            if (!controller.loaded){
                busySpinner.show(false);
            }
        };

        segmentsProxy.getSegments(proj.id, handlePreProcessingJson, null, handlePreProcessingComplete);

        if (!controller.loaded){
            busySpinner.show(true,
                sas.modelfactory.RB.getText("projectProfileView.busyDialog.title.txt"),
                sas.modelfactory.RB.getText("projectProfileView.busyDialog.loadingSegments.txt")
            );
        }
    },

    pollRunningSegments: function(project) {
        var view = this.oView;
        var controller = this;
        var segmentList = view.table.getModel().oData.preData;
        var isRunning = false;
        if (segmentList) {
            for(var i=0;i<segmentList.length;i++){
                if (segmentList[i].status === "running" || segmentList[i].status === "scheduled"){
                    isRunning = true;
                    break;
                }
            }
        }
        // If any segments are RUNNING, poll until finished
        if (isRunning && !controller.stopPolling){
            // Get the segments for the selected segment
            logUtil.log("********** POLLING SEGMENTS", Date(), "**********");
            controller.isPolling = true;
            if (project){
                setTimeout(function(){
                    controller.isPolling = false;
                    if (!controller.stopPolling){
                        controller.getSegments(project);
                    }
                }, controller.CONST_POLL_TIME);
            }
        } else {
            controller.isPolling = false;
        }
    },

    onExcludeAction: function(evt) {
        evt.oSource.setEnabled(false);
        return this.onIncludeExcludeAction(evt, true);
    },

    onIncludeAction: function(evt) {
        evt.oSource.setEnabled(false);
        return this.onIncludeExcludeAction(evt, false);
    },

    onIncludeExcludeAction: function(evt, isExclude) {
        var controller = this;

        var excludeList = [];
        var indices = this.oView.table.getSelectedIndices();
        var context = this.oView.table.getContextByIndex(indices[0]);
        for (var i = 0; i < indices.length; i++) {

            var selectedData = this.oView.table.getContextByIndex(indices[i]).getObject();

            var data = this.oView.table.getModel().oData.preData;
            for (var j=0; j < data.length; j++) {

                // Update each segment(s) excluded bit
                if (data[j].id === selectedData.id) {
                    data[j].excluded = isExclude;
                    data[j].excludedText = isExclude?sas.modelfactory.RB.getText("projectProfileView.menuItem.exclude.txt"):sas.modelfactory.RB.getText("projectProfileView.menuItem.include.txt");
                }
            }
        }

        // Walk through the segments a final time to get the list of currently excluded ones
        try {
            data = JSON.parse(JSON.stringify(this.oView.table.getModel().oData.preData));
        }
        catch(err){
            logUtil.error("Error: Failed to parse JSON stringify data", this.oView.table.getModel().oData.preData);
        }

        for (var j=0; j < data.length; j++) {
            if (data[j].excluded === true) {
                delete data[j].excludedText;
                excludeList.push(data[j]);
            }
        }

        var col = controller.getView().byId("profile_run");
        col.invalidate();
        if (col.getSorted()){
            col.sort(col.getSortOrder());
        }

        var onSuccess = function(oJson){

        };

        var onComplete = function(oJson) {

//            // Retrieve the model again now that the update is done
//            controller.getSegments(controller.project);

            var data = controller.oView.table.getModel().oData.preData;
            var responseJson = oJson.responseJSON;
            var excludedSetting = false;

            // For each segment
            for (var i=0; i < data.length; i++) {
                // Assume not excluded
                excludedSetting = false;

                // Look through the list of excluded segments
                for (var j = 0; j < responseJson.items.length; j++) {
                    // If found the segment, set excluded to true
                    if (data[i].id === responseJson.items[j].id) {
                        excludedSetting = true;
                        break;
                    }
                }
                // Set the excluded property so the model can refresh correctly
                controller.oView.table.getModel().setProperty("/preData/" + i + "/excluded", excludedSetting);
            }

            // Fire a row select for the current row to update the Exclude/Include buttons correctly
            var evt = {rowIndex: null, rowContext: context, rowIndices: indices};
            controller.oView.table.fireRowSelectionChange(evt);
        }

        // Locate the profileId
        var profileId = null;
        if (data.length > 0)
            profileId = data[0].profileId;

        // Call the REST service to update the segments with the excluded list
        projectProxy.updateProjectProfileExcludeList(controller.project.id, profileId, excludeList, onSuccess, null, onComplete);
    },

    onProfileAction: function (evt) {
        var id = evt.getParameter("id");
        logUtil.log("onProfileAction menuItem: ", id);
        var action;
        switch (id)
        {
            case "ProjectProfile--excludeBtn":
                action = "Exclude";
                break;
            case "ProjectProfile--includeBtn":
                action = "Include";
                break;
            case "ProjectProfile--exploreBtn":
                action = "Explore";
                break;
        }

        var tempStr = "";
        var indices = this.oView.table.getSelectedIndices();
        for (var i=0; i<indices.length; i++) {
            var context = this.oView.table.getContextByIndex(indices[i]);
            if (context)
            {
                var selectedData = context.getObject();
                logUtil.log("onProfileAction row selection", selectedData);
                tempStr += selectedData.name + " ";
            }
        }
        sap.m.MessageToast.show(action + " for " + tempStr);
    },

    onColumnMove: function(evt) {
        logUtil.log("Profile table column moved:", evt.getParameter("column").sId, "moved to", evt.getParameter("newPos"));

        // A column has moved
        var params = evt.getParameters();
        this.columnsMoved(params);
    },

    columnsMoved: function(params) {
        logUtil.log("Columns changed.  Do something with columns.")
        if (params) {
            var controller = this;
            var colMoved = params.column;
            var pos = params.newPos;
            var table = controller.getView().table;
            var cols = null;
            if (table){
                cols = table.getColumns();
            }

            // Update the columms
            if (table && cols){
                for(var i=0;i<cols.length;i++){
                    if (cols[i].getId() === colMoved.getId()){
                        cols.splice(i, 1);
                        break;
                    }
                }
                cols.splice(pos, 0, colMoved);
            }
            table.removeAllColumns();
            for(var j=0;j<cols.length;j++){
                table.addColumn(cols[j]);
            }

            // Store column preferences
            var key = "FactoryMiner.ProfileColumnPreferences.";
            if (controller.getTargetType() === "INTERVAL")
                key = key + "Numeric";
            else
                key = key + "Class";

            var storeSuccess = function(args) {
                controller.preferenceId = args.id;
            }

            sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, key, "profile_", table, controller.preferenceId, storeSuccess);
        }
    },

    onColumnManagerClose: function() {
        var controller = this;
        var key = "FactoryMiner.ProfileColumnPreferences.";
        if (controller.getTargetType() === "INTERVAL")
            key = key + "Numeric";
        else
            key = key + "Class";

        var table = controller.getView().table;

        var storeSuccess = function(args) {
            controller.preferenceId = args.id;
        }

        // Store column preferences
        sas.modelfactory.util.PreferencesUtil.setColumnPref(controller.project.id, key, "profile_", table, controller.preferenceId, storeSuccess);
    },

    getTargetType: function() {
        var targetType = "UNDEFINED";
        var targetModel = sap.ui.getCore().getModel("targetType");
        if (targetModel){
            var level = targetModel.getProperty("/level");
            if (level){
                targetType = level.key;
                logUtil.log("Project target type:", targetType);
            }
        }
        return targetType;
    },

    updateMessages: function() {
        // update error message
        var controller = this;
        this.errorMsg = [];

        // Check for any errors...
        if (!controller.profileBuilt) {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.noProfile.txt");
            this.errorMsg.push(errorStr);
        }

        if (this.project.profileStatus.key === "MODIFIED") {
            var errorStr = sas.modelfactory.RB.getText("projectProfileView.error.modifiedProfile.txt");
            this.errorMsg.push(errorStr);
        }

        // Update the error display
        if (this.errorMsg.length  > 0) {
            var errorTxt = this.buildErrorMessage();
            var warningList = this.getView().varList;
            if (warningList){
                warningList.removeAllItems();
                warningList.setVisible(true);
                var list = this.buildErrorList();
                for (var i = 0; i < list.length; i++) {
                    warningList.addItem(list[i]);
                }
            }

            var top = this.getView().pageLayout.getTop();
            var size = this.errorMsg.length * 45;
            size += "px";
            top.setSize(size);
        } else {
            this.getView().varList.setVisible(false);
            var top = this.getView().pageLayout.getTop();
            top.setSize("0px");

            this.getView().varList.setVisible(false);
        }
    },

    buildErrorMessage: function(){
        var controller = this;
        var errorTxt = sas.modelfactory.RB.getText("projectProfileView.errorMsg.txt") + "\n\n";
        for (var i=0;i<this.errorMsg.length;i++){
            errorTxt += "\u2022 " + this.errorMsg[i] + "\n";
        }
        return errorTxt;
    },

    buildErrorList: function(){
        var controller = this;
        var varList = [];
        for (var i=0;i<this.errorMsg.length;i++){
            var item = new sas.hc.m.StandardListItem({
                title: this.errorMsg[i],
                icon: sas.icons.HC.ALERT
            }).addStyleClass("ProjectDataView-warning");
            varList.push(item);
        }
        return varList;
    }
});